@@ -2,13 +2,12 @@ package Blosxom::Header;
 use 5.008_009;
 use strict;
 use warnings;
-use parent qw/Exporter/;
-use overload '%{}' => 'as_hashref', 'bool' => 'boolify', 'fallback' => 1;
-use Blosxom::Header::Adapter;
-use Carp qw/carp croak/;
-use HTTP::Date ();
+use parent 'Blosxom::Header::Hash';
+use Blosxom::Header::Util qw/time2str str2time/;
+use Carp qw/croak carp/;
+use Exporter 'import';
 
-our $VERSION = '0.05009';
+our $VERSION = '0.05010';
 
 our @EXPORT_OK = qw(
     header_get  header_set  header_exists header_delete header_iter
@@ -25,113 +24,27 @@ sub header_iter   { __PACKAGE__->instance->each( @_ )   }
 sub header_push { __PACKAGE__->instance->_push( @_ ) }
 *each_header = \&header_iter;
 
-my %str2time;
-sub _str2time { $str2time{ $_[0] } ||= HTTP::Date::str2time( $_[0] ) }
-
-my %time2str;
-sub _time2str { $time2str{ $_[0] } ||= HTTP::Date::time2str( $_[0] ) }
-
 my $instance;
 
-my %header;
-my $adapter;
-
-sub instance {
+sub new {
     my $class = shift;
 
     return $class if ref $class;
     return $instance if defined $instance;
 
     if ( $class->is_initialized ) {
-        $adapter = tie %header, 'Blosxom::Header::Adapter', $blosxom::header;
-        return $instance = bless \do { my $anon_scalar }, $class;
+        return $instance = $class->SUPER::new( $blosxom::header );
     }
 
     croak( q{$blosxom::header hasn't been initialized yet} );
 }
 
+*instance = \&new;
+
 sub has_instance { $instance }
 
 sub is_initialized { ref $blosxom::header eq 'HASH' }
 
-sub as_hashref { \%header }
-sub boolify { 1 }
-
-sub get {
-    my ( $self, @fields ) = @_;
-    @header{ @fields };
-}
-
-sub set {
-    my ( $self, %new_header ) = @_;
-    @header{ keys %new_header } = values %new_header; # merge!
-    return;
-}
-
-sub delete {
-    my ( $self, @fields ) = @_;
-    delete @header{ @fields };
-}
-
-sub exists {
-    my ( $self, $field ) = @_;
-    exists $header{ $field };
-}
-
-sub clear { %header = () }
-
-sub field_names { $adapter->field_names }
-
-sub each {
-    my ( $self, $callback ) = @_;
-
-    if ( ref $callback eq 'CODE' ) {
-        for my $field ( $adapter->field_names ) {
-            $callback->( $field, $header{ $field } );
-        }
-    }
-    elsif ( defined wantarray ) {
-        return $self->_each;
-    }
-    else {
-        croak( 'Must provide a code reference to each()' );
-    }
-
-    return;
-}
-
-# This method is deprecated and will be remove in 0.06
-my %iterator;
-sub _each {
-    my $self = shift;
-
-    if ( $iterator{is_exhausted} or !%iterator ) {
-        my @fields = $adapter->field_names;
-        %iterator = (
-            collection => \@fields,
-            size       => scalar @fields,
-            current    => 0,
-        );
-    }
-
-    if ( $iterator{current} < $iterator{size} ) {
-        my $field = $iterator{collection}[ $iterator{current}++ ];
-        return wantarray ? ( $field, $header{ $field } ) : $field;
-    }
-    else {
-        $iterator{is_exhausted}++;
-    }
-
-    return;
-}
-
-sub is_empty { not %header }
-
-sub flatten {
-    my $self = shift;
-    map { $_, $header{$_} } $adapter->field_names;
-}
-
 sub set_cookie {
     my ( $self, $name, $value ) = @_;
 
@@ -145,7 +58,7 @@ sub set_cookie {
 
     my @cookies;
     my $offset = 0;
-    if ( my $cookies = $header{Set_Cookie} ) {
+    if ( my $cookies = $self->{Set_Cookie} ) {
         @cookies = ref $cookies eq 'ARRAY' ? @{ $cookies } : $cookies;
         for my $cookie ( @cookies ) {
             last if ref $cookie eq 'CGI::Cookie' and $cookie->name eq $name;
@@ -156,7 +69,7 @@ sub set_cookie {
     # add/overwrite CGI::Cookie object
     splice @cookies, $offset, 1, $new_cookie;
 
-    $header{Set_Cookie} = @cookies > 1 ? \@cookies : $cookies[0];
+    $self->{Set_Cookie} = @cookies > 1 ? \@cookies : $cookies[0];
 
     return;
 }
@@ -165,7 +78,7 @@ sub get_cookie {
     my ( $self, $name ) = @_;
 
     my @values;
-    if ( my $cookies = $header{Set_Cookie} ) {
+    if ( my $cookies = $self->{Set_Cookie} ) {
         @values = grep {
             ref $_ eq 'CGI::Cookie' and $_->name eq $name
         } (
@@ -176,12 +89,28 @@ sub get_cookie {
     wantarray ? @values : $values[0];
 }
 
+# This method is obsolete and will be removed in 0.06
+sub cookie {
+    my $self = shift;
+
+    if ( @_ ) {
+        delete $self->{Set_Cookie};
+        $self->push_cookie( @_ );
+    }
+    elsif ( my $cookie = $self->{Set_Cookie} ) {
+        my @cookies = ref $cookie eq 'ARRAY' ? @{ $cookie } : ( $cookie );
+        return wantarray ? @cookies : $cookies[0];
+    }
+
+    return;
+}
+
 # This method/function is obsolete and will be removed in 0.06
 sub push_cookie {
-    require CGI::Cookie;
-
     my $self = ref $_[0] ? shift : __PACKAGE__->instance;
 
+    require CGI::Cookie;
+
     my @cookies;
     for my $cookie ( @_ ) {
         $cookie = CGI::Cookie->new( $cookie ) if ref $cookie eq 'HASH';
@@ -191,108 +120,34 @@ sub push_cookie {
     $self->_push( Set_Cookie => @cookies );
 }
 
-sub push_p3p_tags {
-    my $self = ref $_[0] ? shift : __PACKAGE__->instance;
-    $self->_push( P3P => @_ );
-}
-
-*push_p3p = \&push_p3p_tags;
-
+# This method is obsolete and will be removed in 0.06
 sub _push {
     my ( $self, $field, @values ) = @_;
 
     return carp( 'Useless use of _push() with no values' ) unless @values;
 
-    if ( my $value = $header{ $field } ) {
+    if ( my $value = $self->{ $field } ) {
         return push @{ $value }, @values if ref $value eq 'ARRAY';
         unshift @values, $value;
     }
 
-    $header{ $field } = @values > 1 ? \@values : $values[0];
+    $self->{ $field } = @values > 1 ? \@values : $values[0];
 
     scalar @values;
 }
 
-sub attachment {
-    my $self = shift;
-    $adapter->attachment( @_ );
-}
-
-sub nph {
-    my $self = shift;
-    $adapter->nph( @_ );
-}
-
 sub charset {
     my $self = shift;
-    my $type = $header{Content_Type};
+    my $type = $self->{Content_Type};
     my ( $charset ) = $type =~ /charset=([^;]+)/ if $type;
     return unless $charset;
     uc $charset;
 }
 
-# This method is obsolete and will be removed in 0.06
-sub cookie {
-    my $self = shift;
-
-    if ( @_ ) {
-        delete $header{Set_Cookie};
-        $self->push_cookie( @_ );
-    }
-    elsif ( my $cookie = $header{Set_Cookie} ) {
-        my @cookies = ref $cookie eq 'ARRAY' ? @{ $cookie } : ( $cookie );
-        return wantarray ? @cookies : $cookies[0];
-    }
-
-    return;
-}
-
-sub last_modified { shift->_date_header( Last_modified => $_[0] ) }
-sub date          { shift->_date_header( Date => $_[0] )          }
-
-sub _date_header {
-    my ( $self, $field, $time ) = @_;
-
-    if ( defined $time ) {
-        return $header{ $field } = _time2str( $time );
-    }
-    elsif ( my $date = $header{ $field } ) {
-        return _str2time( $date );
-    }
-
-    return;
-}
-
-sub expires {
-    my $self = shift;
-    return $header{Expires} = shift if @_;
-    my $expires = $header{Expires};
-    return unless $expires;
-    _str2time( $expires );
-}
-
-sub p3p_tags {
-    my $self = shift;
-
-    if ( @_ ) {
-        my @tags = @_ > 1 ? @_ : split / /, shift;
-        $header{P3P} = @tags > 1 ? \@tags : $tags[0];
-    }
-    elsif ( my $tags = $header{P3P} ) {
-        my @tags = ref $tags eq 'ARRAY' ? @{ $tags } : ( $tags );
-        @tags = map { split / / } @tags;
-        return wantarray ? @tags : $tags[0];
-    }
-
-    return;
-}
-
-*p3p = \&p3p_tags;
-
 sub content_type {
     my $self = shift;
-    return $header{Content_Type} = shift if @_;
-    my $content_type = $header{Content_Type};
+    return $self->{Content_Type} = shift if @_;
+    my $content_type = $self->{Content_Type};
     return q{} unless $content_type;
     my ( $type, $rest ) = split /;\s*/, $content_type, 2;
     wantarray ? ( lc $type, $rest ) : lc $type;
@@ -307,10 +162,10 @@ sub status {
         require HTTP::Status;
         my $code = shift;
         my $message = HTTP::Status::status_message( $code );
-        return $header{Status} = "$code $message" if $message;
+        return $self->{Status} = "$code $message" if $message;
         carp( qq{Unknown status code "$code" passed to status()} );
     }
-    elsif ( my $status = $header{Status} ) {
+    elsif ( my $status = $self->{Status} ) {
         return substr( $status, 0, 3 );
     }
 
@@ -319,10 +174,33 @@ sub status {
 
 sub target {
     my $self = shift;
-    return $header{Window_Target} = shift if @_;
-    $header{Window_Target};
+    return $self->{Window_Target} = shift if @_;
+    $self->{Window_Target};
 }
 
+sub last_modified { shift->_date_header( Last_modified => $_[0] ) }
+sub date          { shift->_date_header( Date => $_[0] )          }
+
+sub _date_header {
+    my ( $self, $field, $time ) = @_;
+
+    if ( defined $time ) {
+        return $self->{ $field } = time2str( $time );
+    }
+    elsif ( my $date = $self->{ $field } ) {
+        return str2time( $date );
+    }
+
+    return;
+}
+
+# The following methods are obsolete and will be removed in 0.06
+sub push_p3p {
+    my $self = ref $_[0] ? shift : __PACKAGE__->instance;
+    $self->push_p3p_tags( @_ );
+}
+*p3p = \&p3p_tags;
+
 1;
 
 __END__
@@ -370,6 +248,7 @@ methods.
 =item $header = Blosxom::Header->instance
 
 Returns a current Blosxom::Header object instance or create a new one.
+C<new()> is an alias.
 
 =item $header = Blosxom::Header->has_instance
 
@@ -401,7 +280,7 @@ You can use underscores as a replacement for dashes in header names.
 
   # field names are case-insensitive
   $header->get( 'Content-Length' );
-  $header->get( 'Content_length' );
+  $header->get( 'content_length' );
 
 =item $header->set( $field => $value )
 
@@ -484,7 +363,10 @@ Returns a Boolean value telling whether C<< $header->field_names >>
 returns a null array or not.
 
   $header->clear;
-  my $is_empty = $header->is_empty; # true
+
+  if ( $header->is_empty ) { # true
+      ...
+  }
 
 =item @headers = $header->flatten
 
@@ -493,7 +375,31 @@ Returns pairs of fields and values.
   my @headers = $header->flatten;
   # => ( 'Status', '304 Not Modified', 'Content-Type', 'text/plain' )
 
-=item $header->as_hashref
+=item $hashref = $header->as_hashref
+
+Returns a reference to hash which represents header fields.
+You can C<set()>, C<get()> and C<delete()> header fields using the hash
+syntax.
+
+  $header->as_hashref->{Foo} = 'bar';
+  my $value = $header->as_hashref->{Foo};
+  my $deleted = delete $header->as_hashref->{Foo};
+
+The hash dereference operator of C<$header> is L<overload>ed with C<as_hashref()>
+and so you can omit calling C<as_hashref()> method:
+
+  $header->{Foo} = 'bar';
+  my $value = $header->{Foo};
+  my $deleted = delete $header->{Foo};
+
+NOTE: You can't iterate over C<$header> using C<CORE::each()>, C<CORE::keys()>
+or C<CORE::values()>. Use C<< $header->field_names >> or C<< $header->each >>
+instead.
+
+  # wrong
+  keys %{ $header };
+  values %{ $header };
+  each %{ $header };
 
 =back
 
@@ -681,17 +587,15 @@ a NPH (no-parse-header) script:
 Represents the P3P tags. C<p3p()> is an alias.
 The parameter can be an array or a space-delimited string.
 
-  $header->p3p( qw/CAO DSP LAW CURa/ );
-  $header->p3p( 'CAO DSP LAW CURa' );
+  $header->p3p_tags( qw/CAO DSP LAW CURa/ );
+  $header->p3p_tags( 'CAO DSP LAW CURa' );
 
-  my @tags = $header->p3p; # ( 'CAO', 'DSP', 'LAW', 'CURa' )
+  my @tags = $header->p3p_tags; # ( 'CAO', 'DSP', 'LAW', 'CURa' )
 
 In this case, the outgoing header will be formatted as:
 
   P3P: policyref="/w3c/p3p.xml" CP="CAO DSP LAW CURa"
 
-=item @tags = $header->push_p3p_tags
-
 =item $header->push_p3p_tags( @tags )
 
 Adds P3P tags to the P3P header.
@@ -699,12 +603,12 @@ C<push_p3p()> is an alias.
 Accepts a list of P3P tags.
 
   # get P3P tags
-  my @tags = $header->p3p; # ( 'CAO', 'DSP', 'LAW' )
+  my @tags = $header->p3p_tags; # ( 'CAO', 'DSP', 'LAW' )
 
   # add P3P tags
-  $header->push_p3p( 'CURa' );
+  $header->push_p3p_tags( 'CURa' );
 
-  @tags = $header->p3p; # ( 'CAO', 'DSP', 'LAW', 'CURa' )
+  @tags = $header->p3p_tags; # ( 'CAO', 'DSP', 'LAW', 'CURa' )
 
 =item $code = $header->status
 
@@ -774,9 +678,7 @@ See L<"HANDLING COOKIES">.
 
 =item push_p3p( @tags )
 
-A shorcut for
-
-  Blosxom::Header->instance->push_p3p( @tags );
+This function is obsolete and will be removed in 0.06.
 
 =item header_iter( \&callback )
 
