@@ -1,6 +1,7 @@
 package Blosxom::Header::Adapter;
 use strict;
 use warnings;
+use Blosxom::Header::Util;
 use Carp qw/carp/;
 use CGI::Util;
 use List::Util qw/first/;
@@ -66,21 +67,21 @@ sub FETCH {
         my $attachment = $adaptee->{-attachment};
         return qq{attachment; filename="$attachment"} if $attachment;
     }
-    elsif ( $norm eq '-expires' ) {
-        my $expires = $adaptee->{ $norm };
-        return $expires ? expires( $expires ) : undef;
-    }
     elsif ( $norm eq '-date' and $self->date_header_is_fixed ) {
-        return expires( time );
+        return Blosxom::Header::Util::expires( time );
+    }
+    elsif ( $norm eq '-p3p' ) {
+        my $p3p = $adaptee->{ $norm };
+        return unless $p3p;
+        my $tags = ref $p3p eq 'ARRAY' ? join ' ', @$p3p : $p3p;
+        return qq{policyref="/w3c/p3p.xml" CP="$tags"};
     }
 
     $adaptee->{ $norm };
 }
 
-*EXISTS = \&FETCH;
 
-my %expires;
-sub expires { $expires{ $_[0] } ||= CGI::Util::expires( $_[0] ) }
+*EXISTS = \&FETCH;
 
 sub STORE {
     my $self    = shift;
@@ -100,6 +101,9 @@ sub STORE {
     elsif ( $norm eq '-date' and $self->date_header_is_fixed ) {
         return carp( 'The Date header is fixed' );
     }
+    elsif ( $norm eq '-p3p' ) {
+        return;
+    }
 
     $adaptee->{ $norm } = $value;
 
@@ -125,6 +129,10 @@ sub DELETE {
     elsif ( $norm eq '-date' and $self->date_header_is_fixed ) {
         return carp( 'The Date header is fixed' );
     }
+    elsif ( $norm eq 'p3p' ) {
+        delete $adaptee->{-p3p};
+        return $self->FETCH( 'P3P' );
+    }
 
     delete $adaptee->{ $norm };
 }
@@ -203,7 +211,43 @@ sub date_header_is_fixed {
     $adaptee->{-expires} || $adaptee->{-cookie} || $adaptee->{-nph};
 }
 
-*has_date_header = \&date_header_is_fixed;
+sub p3p_tags {
+    my $self    = shift;
+    my $adaptee = $self->{adaptee};
+
+    if ( @_ ) {
+        my @tags = @_ > 1 ? @_ : split / /, shift;
+        $adaptee->{-p3p} = @tags > 1 ? \@tags : $tags[0];
+    }
+    elsif ( my $tags = $adaptee->{-p3p} ) {
+        my @tags = ref $tags eq 'ARRAY' ? @{ $tags } : ( $tags );
+        @tags = map { split / / } @tags;
+        return wantarray ? @tags : $tags[0];
+    }
+
+    return;
+}
+
+sub push_p3p_tags {
+    my ( $self, @values ) = @_;
+    my $adaptee = $self->{adaptee};
+
+    if ( my $value = $adaptee->{-p3p} ) {
+        return push @{ $value }, @values if ref $value eq 'ARRAY';
+        unshift @values, $value;
+    }
+
+    $adaptee->{-p3p} = @values > 1 ? \@values : $values[0];
+
+    scalar @values;
+}
+
+sub expires {
+    my $self = shift;
+    my $expires = $self->{adaptee}->{-expires};
+    return unless $expires;
+    Blosxom::Header::Util::expires( $expires );
+}
 
 1;
 
@@ -223,7 +267,7 @@ Blosxom::Header::Adapter - Adapter for CGI::header()
 
   # field names are case-insensitive
   my $length = $adapter{'Content-Length'}; # 1234
-  $adapter{'Content_length'} = 4321;
+  $adapter{'content_length'} = 4321;
 
   print header( %adaptee );
   # Content-length: 4321
@@ -264,6 +308,14 @@ A shortcut for
 
 =item $bool = $adapter->date_header_is_fixed
 
+=item @tags = $adapter->p3p_tags
+
+=item $adapter->p3p_tags( @tags )
+
+=item $adapter->push_p3p_tags( @tags )
+
+=item $date = $adapter->expires
+
 =back
 
 =head1 DIAGONOSTICS
@@ -279,7 +331,6 @@ C<-cookie>, C<-nph> or C<-expires> was set.
 
 =head1 SEE ALSO
 
-L<Blosxom::Header::Iterator>,
 L<Tie::Hash>
 
 =head1 AUTHOR
